#!/bin/sh
# Author      : Ming Lu
# Description : Update FW of TMPM342 
# Create Date : Nov. 30, 2016
# Note        : 
# History:
#     2016/11/30 V1.0.0 - first release
#


#-----------------------------------------------------------------------------
# search contents for building FW image
# Global variables: 
#   1. FW_BIN_FILES: output, file list of FW image of TMPM342
# Input: 
#   1. working directory
#   2. file model name
# Return: none
#
search_bin_contents()
{
	local working_dir=$1
	local file_model_name=$2
	FW_BIN_FILES=""
	
	local bin_files="`ls ${working_dir}/${file_model_name}*.bin 2>/dev/null`"
	local previous_modle_name=""
	local previous_fw_major_ver="0"
	local previous_fw_sub_ver="0"
	local bin_file=""
	
	
	for f in $bin_files
	do
		local filename=`basename $f`
echo "    filename=$filename"
		local modle_name=`echo "$filename" | sed 's/[_,-][v,V][[:digit:]]*[_,-,\.][[:digit:]]*\.[b,B][i,I][n,N]$//g'`
echo "    modle_name=$modle_name"
		local fw_ver_str=`echo "$filename" | sed "s/^${modle_name}[_,-][v,V]//g" | sed 's/\.[b,B][i,I][n,N]$//g' | sed 's/-/_/g' | sed 's/\./_/g'`
echo "    fw_ver_str=$fw_ver_str"
		local fw_major_ver=`echo $fw_ver_str | cut -d '_' -f 1 | sed 's/[[:alpha:]]*//g' | sed 's/^0*//g'`
echo "    fw_major_ver=$fw_major_ver"
		local fw_sub_ver=`echo $fw_ver_str | cut -d '_' -f 2 | sed 's/[[:alpha:]]*//g' | sed 's/^0*//g'`
echo "    fw_sub_ver=$fw_sub_ver"
#		if [ -z "$fw_major_ver" -o -z "$fw_sub_ver" -o -z "$modle_name" ]
		if [ -z "$fw_major_ver" -o -z "$modle_name" ]
		then
			continue
		fi
		
		if [ "$previous_modle_name" = "$modle_name" -a ! -z "$previous_modle_name" ]
		then
			if [ $fw_major_ver -gt $previous_fw_major_ver ] || [ $fw_major_ver -eq $previous_fw_major_ver -a $fw_sub_ver -gt $previous_fw_sub_ver ]
			then
				previous_fw_major_ver=$fw_major_ver
				previous_fw_sub_ver=$fw_sub_ver
				bin_file=$f
			fi
		else
			if [ ! -z "$bin_file" ]
			then
				FW_BIN_FILES=$bin_file
			fi
			
			previous_modle_name=$modle_name
			previous_fw_major_ver=$fw_major_ver
			previous_fw_sub_ver=$fw_sub_ver
			bin_file=$f
		fi
	done 

echo "    FW_BIN_FILES=$FW_BIN_FILES"

	if [ ! -z "$bin_file" ]
	then
		FW_BIN_FILES=$bin_file
echo "    FW_BIN_FILES=$FW_BIN_FILES"
	fi
}
#-----------------------------------------------------------------------------
# check FW version of TMPM342
# Global variables: none
# Input: 
#   1. UART ID
# Return: 
#    CUR_FW_VER_STR
#
check_tmpm242_fw_ver()
{
	local uart_id=$1
	local fw_response_str=""
	
	CUR_FW_VER_STR=""
	for b in 19200 9600 19200 115200
	do
		fw_response_str=`/home/lilin/test_485 -b${b} -c${uart_id} -p0  --visca 81090002FF | /bin/grep 'Received data' | /usr/bin/cut -d '=' -f 2 | /bin/sed 's/[[:space:]]*//g'`

		if [ ! -z "$fw_response_str" ]
		then
			local fw_model_str=`echo "$fw_response_str" | /usr/bin/cut -c 9-12`
			local fw_major_str=`echo "$fw_response_str" | /usr/bin/cut -c 13-14`
			local fw_sub_str=`echo "$fw_response_str" | /usr/bin/cut -c 15-16`
			if [ ! -z "${fw_major_str}" -a ! -z "${fw_sub_str}" ]
			then
				CUR_FW_VER_STR=`/usr/bin/printf "%d.%d" 0x${fw_major_str} 0x${fw_sub_str}`
				break
			fi
		fi		
	done
	
	if [ -z "$CUR_FW_VER_STR" ]
	then
		CUR_FW_VER_STR="0.0"
	fi
}

#-----------------------------------------------------------------------------
# Extract tar 
# Global variables: none
# Input:
#   1. tar file
#   2. output path
#   3. temporary folder . The files of tar will be extracted to temporary folder in first if the value is not empty.
# Return: none
#
extract_bin_file()
{
	local bin_file=$1
	local output_dir=$2
	local tmp_dir=$3
	
	echo "    Attach MTD${tmp_dir} ..."
	mkdir -p $output_dir 2>/dev/null
	rm -rf ${output_dir}/* 2>/dev/null
	if [ ! -z "$tmp_dir" ]
	then 
		echo "    Extracting to tmp dir:${tmp_dir}"
		mkdir -p $tmp_dir 2>/dev/null
		rm -rf "${tmp_dir}/*" 2>/dev/null
		tar -xf $bin_file -C $tmp_dir
		
		echo "    Moving to files: From ${tmp_dir} to ${output_dir}"
		mv -f ${tmp_dir}/* ${output_dir}
	else
		tar -xf $bin_file -C $output_dir
	fi
}
#-----------------------------------------------------------------------------

NO_EXTRECT_FALG=1

if [ "$1" = "Extracted" ]
then
	NO_EXTRECT_FALG=0
fi

WORKING_DIR=$2

if [ -z "$WORKING_DIR" ]
then
	WORKING_DIR=/tmp/fw_dir/m342_fw
	
	if [ -d /tmp/fw_files/m342_fw ]
	then
		WORKING_DIR=/tmp/fw_files/m342_fw
	fi
fi
UPLOAD_BIN_FILE=/tmp/upload.bin
NVRAM_NOW=/www/nvram.now
NVRAM_MAC=/etc/ipcam/nvram/nvram.mac

FW_UPGRADE_UART_DEV=""
FW_UPGRADE_GPIO_RESET=""
FW_UPGRADE_GPIO_TYPE=""
FW_UPGRADE_GPIO_BOOT_MODE=""
FW_BIN_FILENAME_STR=""
WCP_FILENAME=""

killall -9 lilin_monitor.sh 2>/dev/null
killall -9 network_recover.sh 2>/dev/null
killall -9 sddpd 2>/dev/null
killall -9 lilin_upnp 2>/dev/null
killall -9 ntpclient 2>/dev/null
killall -9 lilin_ping.sh 2>/dev/null
killall -9 ipcam_service

if [ "$NO_EXTRECT_FALG" = "1" ]
then
	extract_bin_file $UPLOAD_BIN_FILE $WORKING_DIR
fi

#---- Check FW image exist or not
echo 
echo "==> Check FW image exist or not"
if [ -f "${NVRAM_MAC}" ]
then
	MODEL_TYPE=`grep "^MODEL_TYPE=" $NVRAM_MAC | cut -d '=' -f 2`
else
	MODEL_TYPE="0"
fi

if [ "$MODEL_TYPE" = "0" ]
then
	echo "Don't support update FW of TMPM342 when model no is empty or zero"
	if [ "$NO_EXTRECT_FALG" = "1" ]
	then
		sleep 1
echo		"reboot"
	else
		exit 1
	fi
fi

UART_ID=""
#Please update below setting if you want to support new model.
case "$MODEL_TYPE" in
	"196" )
		FW_BIN_FILENAME_STR="ZR8022_20X"
		WCP_FILENAME="Common256_UART.wcp"

		UART_ID=1
		FW_UPGRADE_GPIO_RESET=92
		FW_UPGRADE_GPIO_BOOT_MODE=94		
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"195" )
		FW_BIN_FILENAME_STR="ZR8022_10X"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=1
		FW_UPGRADE_GPIO_RESET=92
		FW_UPGRADE_GPIO_BOOT_MODE=94		
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"220" )
		FW_BIN_FILENAME_STR="PZD6422_EX3"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=1
		FW_UPGRADE_GPIO_RESET=36
		FW_UPGRADE_GPIO_BOOT_MODE=4
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"545"|"559" )
		FW_BIN_FILENAME_STR="ZR8022_25X"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=1
		FW_UPGRADE_GPIO_RESET=5
		FW_UPGRADE_GPIO_BOOT_MODE=1		
		FW_UPGRADE_GPIO_TYPE=1
		;;
	"266"|"270"|"291"|"292"|"659"|"715" )
		FW_BIN_FILENAME_STR="cv22_25X"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=3
		FW_UPGRADE_GPIO_RESET=32
		FW_UPGRADE_GPIO_BOOT_MODE=30
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"262"|"268"|"272"|"714"|"716" )
		FW_BIN_FILENAME_STR="cv22_30X"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=3
		FW_UPGRADE_GPIO_RESET=32
		FW_UPGRADE_GPIO_BOOT_MODE=30
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"273"|"274"|"277"|"278"|"281"|"285"|"286"|"287"|"288"|"289"|"290" )
		FW_BIN_FILENAME_STR="cv22_25X"
		WCP_FILENAME="Common256_UART.wcp"

		UART_ID=4
		FW_UPGRADE_GPIO_RESET=44
		FW_UPGRADE_GPIO_BOOT_MODE=43
		FW_UPGRADE_GPIO_TYPE=0
		;;
	"275"|"276"|"279"|"280" )
		FW_BIN_FILENAME_STR="cv22_30X"
		WCP_FILENAME="Common256_UART.wcp"
		
		UART_ID=4
		FW_UPGRADE_GPIO_RESET=44
		FW_UPGRADE_GPIO_BOOT_MODE=43
		FW_UPGRADE_GPIO_TYPE=0
		;;
	*)
		echo "Don't support model type: $MODEL_TYPE"
		if [ "$NO_EXTRECT_FALG" = "1" ]
		then
			sleep 1
echo		"reboot"
		else
			exit 1
		fi
		;;
esac

echo "    MODEL_TYPE=$MODEL_TYPE"
echo "    FW_BIN_FILENAME_STR=$FW_BIN_FILENAME_STR"

FW_UPGRADE_UART_DEV="/dev/ttyS${UART_ID}"

#M342_FW_BIN_FILENAME=`ls ${WORKING_DIR}/${FW_BIN_FILENAME_STR}*.bin 2>/dev/null | xargs basename`

search_bin_contents "${WORKING_DIR}" "${FW_BIN_FILENAME_STR}"
M342_FW_BIN_FILENAME=`basename $FW_BIN_FILES`
if [ -z "$M342_FW_BIN_FILENAME" -o -z "$WCP_FILENAME" -o -z "$UART_ID" ] || [ ! -f "${WORKING_DIR}/WCP/${WCP_FILENAME}" ] || [ ! -f "${WORKING_DIR}/${M342_FW_BIN_FILENAME}" ]
then
	echo "  FW image of TMPM342 not include in FW image, file name=$FW_BIN_FILENAME_STR, M342_FW_BIN_FILENAME=$M342_FW_BIN_FILENAME"
	echo "  Or WCP (programming routine) file is empty/not exist. WCP_FILENAME=$WCP_FILENAME, WORKING_DIR=${WORKING_DIR}"
	if [ "$NO_EXTRECT_FALG" = "1" ]
	then
		sleep 1
echo		"reboot"
	else
		exit 1
	fi
fi

#---- Current FW version
echo 
echo "==> Checking current FW version"

if [ -f "${NVRAM_NOW}" ]
then
	#CUR_FW_VER_STR=`grep "^M342VERSION=" $NVRAM_NOW | cut -d '=' -f 2 | sed 's/[a-z,A-Z]//g'`
	check_tmpm242_fw_ver ${UART_ID}
else
	CUR_FW_VER_STR="0.0"
fi

CUR_FW_MAJOR_VER=`echo $CUR_FW_VER_STR | cut -d '.' -f 1 | sed 's/^0*//g'`
CUR_FW_SUB_VER=`echo $CUR_FW_VER_STR | cut -d '.' -f 2 | sed 's/^0*//g'`

IMG_FW_VER_STR=`echo $M342_FW_BIN_FILENAME | sed "s/^${FW_BIN_FILENAME_STR}[_,-][v,V]//g" | sed 's/\.bin$//g' | sed 's/-/_/g' | sed 's/\./_/g'`
IMG_FW_MAJOR_VER=`echo $IMG_FW_VER_STR | cut -d '_' -f 1 | sed 's/[[:alpha:]]*//g' | sed 's/^0*//g'`
IMG_FW_SUB_VER=`echo $IMG_FW_VER_STR | cut -d '_' -f 2 | sed 's/[[:alpha:]]*//g' | sed 's/^0*//g'`

echo "  Current FW of TMPM342 version: ${CUR_FW_MAJOR_VER}.${CUR_FW_SUB_VER}  / ${CUR_FW_VER_STR}"
echo "  FW image version: ${IMG_FW_MAJOR_VER}.${IMG_FW_SUB_VER}  / ${IMG_FW_VER_STR}"

#if [ -z "$IMG_FW_MAJOR_VER" -o -z "$IMG_FW_SUB_VER" -o -z "$CUR_FW_MAJOR_VER" -o -z "$CUR_FW_SUB_VER" ]
if [ -z "$IMG_FW_MAJOR_VER" -o -z "$IMG_FW_SUB_VER" ]
then
	echo "   Unknown FW version"
	if [ "$NO_EXTRECT_FALG" = "1" ]
	then
		sleep 1
echo		"reboot"
	else
		exit 1
	fi
fi

#if [ $CUR_FW_MAJOR_VER -gt $IMG_FW_MAJOR_VER ] || [ $IMG_FW_MAJOR_VER -eq $CUR_FW_MAJOR_VER -a $CUR_FW_SUB_VER -ge $IMG_FW_SUB_VER ]
#then
#	echo ""
#	echo ""
#	echo "  WARNING!! You are updating with previous FW version image!!"
#	echo "            FW verions of image: ${IMG_FW_MAJOR_VER}.${IMG_FW_SUB_VER} , current FW: ${CUR_FW_MAJOR_VER}.${CUR_FW_SUB_VER}"
#	echo ""
#	if [ "$NO_EXTRECT_FALG" = "1" ]
#	then
#		sleep 5
#echo		"reboot"
#	else
#		exit 1
#	fi
#fi

echo "==> done"
echo 

#---- check image

echo "==> Checking FW imgae .."

WCP_CHECKSUM_VAL=`grep ^${WCP_FILENAME} ${WORKING_DIR}/WCP/md5.log | cut -d '=' -f 2`
FW_CHECKSUM_VAL=`grep ^${M342_FW_BIN_FILENAME} ${WORKING_DIR}/md5.log | cut -d '=' -f 2`

if [ -z "$WCP_CHECKSUM_VAL" ]
then
	WCP_CHECKSUM_VAL=`grep ^${WCP_FILENAME} ${WORKING_DIR}/WCP/md5.log | cut -d ' ' -f 2`
fi

if [ -z "$FW_CHECKSUM_VAL" ]
then
	FW_CHECKSUM_VAL=`grep ^${M342_FW_BIN_FILENAME} ${WORKING_DIR}/md5.log | cut -d '=' -f 2`
fi

MD5_WCP=`md5sum ${WORKING_DIR}/WCP/${WCP_FILENAME} | cut -d ' ' -f 1`
MD5_FW_BIN=`md5sum ${WORKING_DIR}/${M342_FW_BIN_FILENAME} | cut -d ' ' -f 1`

if [ ! "$WCP_CHECKSUM_VAL" = "$MD5_WCP" -o ! "$FW_CHECKSUM_VAL" = "$MD5_FW_BIN" ]
then
	echo ""
	echo ""
	echo "  WARNING!! MD5 is incorrect!!"
	echo "            $WCP_FILENAME: $MD5_WCP / $WCP_CHECKSUM_VAL"
	echo "            $M342_FW_BIN_FILENAME: $MD5_FW_BIN / $FW_CHECKSUM_VAL"
	echo ""
	if [ "$NO_EXTRECT_FALG" = "1" ]
	then
		sleep 5
echo		"reboot"
	else
		exit 1
	fi
fi

echo "==> done"
echo 

#---- update FW
if [ "$MODEL_TYPE" = "220" ]
then
	echo "==> Bypass Pan-Tilt.."
	/home/lilin/test_485 -b19200 --visca E18AE301006E
fi

echo "==>updating FW of TMPM342.."

chmod a+x ${WORKING_DIR}/tmpm342_fw

echo "${WORKING_DIR}/tmpm342_fw -u ${FW_UPGRADE_UART_DEV} -g ${FW_UPGRADE_GPIO_TYPE} -r ${FW_UPGRADE_GPIO_RESET} -b ${FW_UPGRADE_GPIO_BOOT_MODE} -w ${WORKING_DIR}/WCP/${WCP_FILENAME} -f ${WORKING_DIR}/${M342_FW_BIN_FILENAME}"

${WORKING_DIR}/tmpm342_fw -u ${FW_UPGRADE_UART_DEV} -g ${FW_UPGRADE_GPIO_TYPE} -r ${FW_UPGRADE_GPIO_RESET} -b ${FW_UPGRADE_GPIO_BOOT_MODE} -w ${WORKING_DIR}/WCP/${WCP_FILENAME} -f ${WORKING_DIR}/${M342_FW_BIN_FILENAME}

if [ "$MODEL_TYPE" = "220" ]
then
	echo "==> Reset Pan-Tilt.."
	/usr/local/bin/amba_debug -g 38 -d 0
fi

echo "==> done"
echo 

sync

if [ "$NO_EXTRECT_FALG" = "1" ]
then
	reboot
else
	exit 0
fi
