#!/bin/sh

UPLOAD_BIN_FILE=/tmp/upload.bin
TMP_DIR=/tmp/fw_dir
NVCLIENT_UTL=/usr/local/bin/nvclient
FLASH_ERASEALL_UTLS=/usr/sbin/flash_eraseall
UPGRADE_PARTITION_UTLS=/usr/local/bin/upgrade_partition
NANDWRITE_UTLS=/usr/sbin/nandwrite

readonly HW_CONF=/etc/lilin_hw.conf

#-----------------------------------------------------------------------------
# Get NVRAM data from nvramd
# Global variables: 
#   1. NVRAM_DATA
#   2. NVCLIENT_UTL
# Input: none
#   1. key of nvramd data
# Return:
#   0: success
#   1: key not exist or nvramd not exist
#
common_get_nvram_data()
{
	local result=1
	local nvram_key=$1
	
	NVRAM_DATA=""
	if [ ! -z "$nvram_key" ]; then
		local result_str=`${NVCLIENT_UTL} -k "$nvram_key" -g`
		local ret=`echo "$result_str" | grep 'Get nvram data, ret' | cut -d ':' -f 2 | tr -d "\r" | tr -d "\n"`
		
		if [ "$ret" = "0" ]; then
			result=0
			NVRAM_DATA=`echo "$result_str" | grep 'Data=' | cut -d '=' -f 2- | tr -d "\r" | tr -d "\n"`
		fi
	fi
	return $result
}
#-----------------------------------------------------------------------------
# update kernel 
# Global variables: 
#   FLASH_ERASEALL_UTLS
#   UPGRADE_PARTITION_UTLS
# Input:
#   1. image file
#   2. MTD dev
# Return: 
#   On success, zero is returned. Otherwise, 1 is returned.
#
update_kernel()
{
	local image_file=$1
	local kernel_mtd=$2
	if [ ! -f "$image_file" -o -z "$kernel_mtd" ] 
	then
		return 1
	fi
	
	$FLASH_ERASEALL_UTLS $kernel_mtd
	$NANDWRITE_UTLS -p $kernel_mtd $image_file
	sync

	return 0
}
#-----------------------------------------------------------------------------
# update kernel_emmc 
# Global variables: 
#   FLASH_ERASEALL_UTLS
#   UPGRADE_PARTITION_UTLS
# Input:
#   1. image file
#   2. eMMC dev
# Return: 
#   On success, zero is returned. Otherwise, 1 is returned.
#
update_kernel_emmc()
{
	local image_file=$1
	local kernel_dev=$2
	if [ ! -f "$image_file" -o -z "$kernel_dev" ] 
	then
		return 1
	fi
	
	#$FLASH_ERASEALL_UTLS $kernel_mtd
	$UPGRADE_PARTITION_UTLS -p -K $kernel_dev $image_file
	#$NANDWRITE_UTLS -p $kernel_mtd $image_file
	#sync

	return 0
}
#-----------------------------------------------------------------------------
# mount ramfs 
# Global variables: none
# Input:
#   1. mount point
# Return: 
#   On success, zero is returned. Otherwise, 1 is returned.
#
mount_ramfs()
{
	local mount_dir=$1
	
	mkdir -p $mount_dir 2>/dev/null
	
	local mpoint_str=`mount | grep "\ on\ $mount_dir"`
	if [ -z "$mpoint_str" ]
	then
		mount -t ramfs ramfs $mount_dir 2>/dev/null
		mpoint_str=`mount | grep "ramfs\ on\ $mount_dir"`
	else
		mpoint_str=`mount | grep "ramfs\ on\ $mount_dir"`
	fi
	
	if [ -z "$mpoint_str" ]
	then
		return 1
	else
		return 0
	fi
}
#-----------------------------------------------------------------------------
restart_nvramd()
{
	/etc/init.d/S21nvramd stop
	/etc/init.d/S21nvramd start
}
#-----------------------------------------------------------------------------

/etc/init.d/S90ipcam_service stop
restart_nvramd

SN_STR=`$UPGRADE_PARTITION_UTLS --show_info | grep ^sn[[:space:]]*= | cut -d '=' -f 2 | sed 's/^[[:space:]]*//g'`

UPDATE_RESULT=${SN_STR:0:1}
KERNEL_NO=${SN_STR:1:1}
SN_OTHER_STR=${SN_STR:2}

if [ -z "$UPDATE_RESULT" ]
then
	UPDATE_RESULT="0"
fi

if [ -z "$KERNEL_NO" ]
then
	KERNEL_NO="0"
fi

if [ -z "$SN_OTHER_STR" ]
then
	SN_OTHER_STR="000000"
fi

# 0 : current kernel image in PRI
# 1 : current kernel image in SEC
if [ "$KERNEL_NO" = "0" ]
then
	MTD=`grep pri /proc/mtd | cut -d ':' -f 1`
else
	MTD=`grep sec /proc/mtd | cut -d ':' -f 1`
fi
KERNEL_MTD_DEV="/dev/$MTD"

INIT_ROOT_DEVICE=$(cat /proc/cmdline | sed -e "s/^.*root=//" -e "s/ .*$//")
KERNEL_DEVICE="/dev/mmcblk0p6"

#---- Current FW version
echo 
echo "==> Checking current FW version"

HW_SDK_VERSION="10.1"

common_get_nvram_data "VERSION"
CURRENT_FW_VERSION=`echo "$NVRAM_DATA" | grep "^${HW_SDK_VERSION}." 2>/dev/null |  cut -d '.' -f 3 2>/dev/null`

#CPIO_ROOTFS=`grep CPIO_ROOTFS $HW_CONF | cut -d '=' -f 2`

#if [ -z "$CURRENT_FW_VERSION" ] || [ "$CPIO_ROOTFS" = "y" -o "$CPIO_ROOTFS" = "Y" ]; then
#	echo "    INVALID VERSION FOR THIS UPDATE!!! CPIO=$CPIO_ROOTFS, FW_VERSION=$NVRAM_DATA, CURRENT_FW_VERSION=$CURRENT_FW_VERSION"
#	echo "    REBOOTING and discard upgrade!!!!!"
#	/etc/init.d/S21nvramd stop
#	sleep 5
#	reboot &
#	exit 1
#fi

echo "==> current FW version: ${CURRENT_FW_VERSION} is OK for upgrading"

#BUILD_SPD_MODEL
common_get_nvram_data "MODEL_TYPE"
MODEL_TYPE=${NVRAM_DATA}

/etc/init.d/S21nvramd stop &

echo "==> Model type: ${MODEL_TYPE}"

#decompress tar file
echo "==============================="
echo "  ==> Decompressing FW image ...."

mount_ramfs ${TMP_DIR}
mkdir -p ${TMP_DIR}
/bin/tar -C ${TMP_DIR} -xf ${UPLOAD_BIN_FILE} md5.log
/bin/tar -C ${TMP_DIR} -xf ${UPLOAD_BIN_FILE} updateam.bin

#check file integrity
echo "==============================="
echo "  ==> Checking FW image integrity ... "

MD5_CONF=`grep updateam.bin ${TMP_DIR}/md5.log | cut -d ' ' -f 1 | sed "s/[[:space:]]//g" | awk '{print toupper($0)}'`
FILE_MD5=`md5sum ${TMP_DIR}/updateam.bin | cut -d ' ' -f 1 | sed "s/[[:space:]]//g" | awk '{print toupper($0)}'`

if [ -z "$MD5_CONF" -o -z "$FILE_MD5" -o ! "$MD5_CONF" = "$FILE_MD5" ]
then
	echo "    Error!! FW image MD5 value is not matched, CONFIG=$MD5_CONF, updateam.bin=$FILE_MD5, reboot late"
	sleep 5
	reboot &
	exit 1
fi
echo "  ==> passed"

rm -r /www/js /www/icon
rm /www/*.htm

/bin/tar -C ${TMP_DIR} -xf ${UPLOAD_BIN_FILE} Image
/bin/tar -C /www -xvf ${UPLOAD_BIN_FILE} video.cab
/bin/tar -C ${TMP_DIR} --exclude="md5.log" --exclude="updateam.bin" --exclude="Image" --exclude="*.cab" -xf ${UPLOAD_BIN_FILE}
rm ${UPLOAD_BIN_FILE}

echo "  ==> done"

#---- update kernel
kernel_img=${TMP_DIR}/Image
if [ -f "$kernel_img" ]
then
	echo 
	if [ -z "${INIT_ROOT_DEVICE}" ]
	then
	    echo "==> Updating kernel Image start (${KERNEL_MTD_DEV})"
	
	    update_kernel ${kernel_img} ${KERNEL_MTD_DEV}
	else
	    echo "==> Updating kernel eMMC Image ${kernel_img} start (${INIT_ROOT_DEVICE})"
	    update_kernel_emmc ${kernel_img} ${KERNEL_DEVICE}
	fi
	
	if [ ! "$?" = "0" ]
	then
		#keep original SN if update kernel fail
		echo "    Error!! update kernel fail"
		echo "    Error!! update kernel fail"
		echo "    Error!! update kernel fail"
		sync
		sleep 5 
		reboot &
		exit 1		
	fi
	
	echo "==> Updated kernel Image done"
	echo 
fi


#---- m342_fw
#BUILD_SPD_MODEL

FEATURE_SYS_PTZ=`grep FEATURE_SYS_PTZ= /etc/ipcam/default_nvram/model.${MODEL_TYPE}.conf 2>/dev/null | sed 's/[[:space:]]//g' | cut -d '=' -f 2`

if [ "$FEATURE_SYS_PTZ" = "1" -a -f ${TMP_DIR}/m342_fw/update_tmpm342.sh ]
then
	echo 
	echo "==> Update TMPM342 FW start"
	chmod a+x ${TMP_DIR}/m342_fw/update_tmpm342.sh
	${TMP_DIR}/m342_fw/update_tmpm342.sh Extracted
	
	echo 
	echo "    update TMPM342 result=$?"
	echo 
fi
#BUILD_SPD_MODEL

#update files
echo "==============================="
echo "  ==> Updating FW ... "

echo "    extracting busybox"
/bin/tar -C ${TMP_DIR} -xvf ${TMP_DIR}/updateam.bin ./bin/busybox

echo "    updating all files.."
${TMP_DIR}/bin/busybox tar -C / -xvf ${TMP_DIR}/updateam.bin
sync

if [ -f "/snmp.bin" ] 
then
	echo "    Udate SNMP package"
	tar -C / -xf /snmp.bin
	rm -f /snmp.bin
fi
sync

echo "  ==> done"

reboot -n -f &

exit 0

